@@ -1,7 +1,7 @@
 
 # C3 private & public binary block structure saving, loading & validation
 
-import copy, binascii, datetime
+import copy, binascii, time, struct, os
 
 from c3.constants import *
 from c3.errors import StructureError, IntegrityError, CertExpired
@@ -21,12 +21,12 @@ class AttrDict(dict):
 
 def make_priv_block(priv_bytes, bare=False):
     privd = AttrDict()
-    privd["key_type"] = KEYTYPE_ECDSA_256P
+    privd["key_type"] = KT_ECDSA_PRIME256V1
     privd["priv_type"] = PRIVTYPE_BARE if bare else PRIVTYPE_PASS_PROTECT
     privd["priv_data"] = priv_bytes
     privd["crc32"] = binascii.crc32(privd.priv_data, 0) % (1 << 32)
-    out_bytes = b3.schema_pack(PRIV_CRCWRAPPED, privd)
-    out_bytes_with_hdr = b3.encode_item_joined(KEY_PRIV_CRCWRAPPED, b3.DICT, out_bytes)
+    out_bytes = b3.schema_pack(PRIV_CRCWRAP_SCHEMA, privd)
+    out_bytes_with_hdr = b3.encode_item_joined(PRIV_CRCWRAPPED, b3.DICT, out_bytes)
     return out_bytes_with_hdr
 
 
@@ -36,10 +36,10 @@ def make_priv_block(priv_bytes, bare=False):
 # Caller must then decrypt the private key if needed.
 
 def load_priv_block(block_bytes):
-    _, index = expect_key_header([KEY_PRIV_CRCWRAPPED], b3.DICT, block_bytes, 0)
-    privd = AttrDict(b3.schema_unpack(PRIV_CRCWRAPPED, block_bytes[index:]))
+    _, index = expect_key_header([PRIV_CRCWRAPPED], b3.DICT, block_bytes, 0)
+    privd = AttrDict(b3.schema_unpack(PRIV_CRCWRAP_SCHEMA, block_bytes[index:]))
     # --- Sanity checks ---
-    schema_ensure_mandatory_fields(PRIV_CRCWRAPPED, privd)
+    schema_ensure_mandatory_fields(PRIV_CRCWRAP_SCHEMA, privd)
     if privd.priv_type not in KNOWN_PRIVTYPES:
         raise StructureError("Unknown privtype %d in priv block (wanted %r)" % (privd.priv_type, KNOWN_PRIVTYPES))
     if privd.key_type not in KNOWN_KEYTYPES:
@@ -55,27 +55,33 @@ def load_priv_block(block_bytes):
 
 # In: public_part bytes
 # Out: data-and-sig list cert chain structure
-# Note: the inverse of this function is part of make_sign
+# Note: the inverse of this function is part of sign
 
 def load_pub_block(public_part):
     # The public part should have an initial header that indicates whether the first das is a payload or a cert
-    ppkey, index = expect_key_header([KEY_LIST_PAYLOAD, KEY_LIST_CERTS], b3.LIST, public_part, 0)
+    ppkey, index = expect_key_header([PUB_CSR, PUB_PAYLOAD, PUB_CERTCHAIN], None, public_part, 0)
     public_part = public_part[index:]               # chop off the header
 
+    # PUB_CSR is just a cert by itself
+    if ppkey == PUB_CSR:
+        cert = AttrDict(b3.schema_unpack(CERT_SCHEMA, public_part))
+        return ppkey, cert
+    # PUB_PAYLOAD and PUB_CERTCHAIN are chains.
     # Should be a list of DAS structures, so pythonize the list
-    chain = list_of_schema_unpack(DATA_AND_SIG, [KEY_DAS], public_part)
+
+    chain = list_of_schema_unpack(DATASIG_SCHEMA, [HDR_DAS], public_part)
 
     # unpack the certs & sigs in chain
     for i, das in enumerate(chain):
-        # dont unpack cert if this is the first das and ppkey is PAYLOAD
-        if i > 0 or ppkey == KEY_LIST_CERTS:
+        # dont unpack cert dif this is the first das and ppkey is PAYLOAD
+        if i > 0 or ppkey == PUB_CERTCHAIN:
             das["cert"] = AttrDict(b3.schema_unpack(CERT_SCHEMA, das.data_part))
             schema_ensure_mandatory_fields(CERT_SCHEMA, das.cert)
 
         das["sig"] = AttrDict(b3.schema_unpack(SIG_SCHEMA, das.sig_part))
         schema_ensure_mandatory_fields(SIG_SCHEMA, das.sig)
 
-    return chain
+    return ppkey, chain
 
 
 # --- Structure helper functions ---
@@ -123,10 +129,10 @@ def schema_ensure_mandatory_fields(schema, dx):
                 raise StructureError("Mandatory field '%s' is %r" % (field_name, dx[field_name]))
 
 
-
 # Index and Unicode are the only two unhandled exception types that b3's decode_header code produces when fuzzed.
 # IndexError trying to decode a bad varint for ext_type, datalen or number key.
 # Unicode for when b3 thinks there's a utf8 key but the utf8 is bad.
+
 def expect_key_header(want_keys, want_type, buf, index):
     if not buf:
         raise StructureError("No data - buffer is empty or None")
@@ -138,7 +144,7 @@ def expect_key_header(want_keys, want_type, buf, index):
     if key not in want_keys:
         raise StructureError \
             ("Incorrect key in header - wanted %r got %s" % (want_keys, repr(key)[:32]))
-    if data_type != want_type:
+    if want_type is not None and want_type != data_type:   # note often we dont want type checking
         raise StructureError \
             ("Incorrect type in header - wanted %r got %s" % (want_type, repr(data_type)[:32]))
     if not has_data:
@@ -147,29 +153,23 @@ def expect_key_header(want_keys, want_type, buf, index):
         raise StructureError("No data after header - buffer is empty")
     return key, index
 
+# Used by the uniloader when someone passes a binary block directly.
 
-# This does not ensure mandatory fields are present like load_pub_block() does, so it can be used for
-# more things e.g. friendly_fields and check_expiry.
-
-def extract_first_dict(part_block, schema):
-    if schema == PRIV_CRCWRAPPED:  # private part block
-        ppkey, index = expect_key_header([KEY_PRIV_CRCWRAPPED, ], b3.DICT, part_block, 0)
-        private_part = part_block[index:]
-        dx0 = AttrDict(b3.schema_unpack(schema, private_part))
-    else:           # public part block, using CERT_SCHEMA or a caller's SCHEMA
-        ppkey, index = expect_key_header([KEY_LIST_PAYLOAD, KEY_LIST_CERTS], b3.LIST, part_block, 0)
-        public_part = part_block[index:]
-        das0 = list_of_schema_unpack(DATA_AND_SIG, [KEY_DAS], public_part)[0]
-        dx0 = AttrDict(b3.schema_unpack(schema, das0.data_part))
-    return dx0
-
-def ensure_not_expired(using_pub):
-    dx0 = extract_first_dict(using_pub, CERT_SCHEMA)
-    expiry = dx0["expiry_date"]
-    if datetime.date.today() > expiry:
-        raise CertExpired("cert specified by --using has expired")
-    return True
+def split_binary_pub_priv(block_in):
+    want_keys = [PUB_CSR, PUB_PAYLOAD, PUB_CERTCHAIN, PRIV_CRCWRAPPED, DUALBLOCK]
+    key, index = expect_key_header(want_keys, None, block_in, 0)
+    if key == PRIV_CRCWRAPPED:
+        return b"", block_in
+    if key in (PUB_CSR, PUB_PAYLOAD, PUB_CERTCHAIN):
+        return block_in, b""
+    if key == DUALBLOCK:
+        dual = b3.schema_unpack(DUALBLOCK_SCHEMA, block_in[index:])
+        return dual["public"], dual["private"]
 
+def combine_binary_pub_priv(pub_block, priv_block):
+    dbx = dict(public=pub_block, private=priv_block)
+    dual_bytes = b3.schema_pack(DUALBLOCK_SCHEMA, dbx)
+    return b3.encode_item_joined(DUALBLOCK, b3.DICT, dual_bytes)
 
 
 # --- Output/Results fetchers ---
@@ -217,42 +217,35 @@ def ctnm(das):
 
 # --------------------------------------------------------------------------------------------------
 
+# ULID generator that doesn't care about trying to be sub-millisecond monotonic.
+def gen_ulid():
+    stamp_bytes = struct.pack(">Q", int(time.time() * 1000.0))[2:]
+    rand_bytes = os.urandom(10)
+    return stamp_bytes + rand_bytes
 
-# Making ULIDS
+
+# Note: a better approach here is to use micro or nanosecond timers.
+# This guy has a number of good strategies:  https://github.com/RobThree/NUlid/tree/master/NUlid/Rng
+# random vs monotonic is a tradeoff-spectrum, with cryptographically-secure RNG at one and and
+# "just use DB autoincrement fields instead" at the other.
+
+
+# ULID gen using super basic "keep trying random numbers" monotonic-increase tekneeq
+# Dont want it sucking systems dry of urandom though.
+# prev_rand = None
+# prev_stamp = None
+# def gen_ulid():
+#     global prev_rand, prev_stamp
+#     nn = 0
+#     stamp = int(time.time() * 1000.0)
+#     stamp_bytes = struct.pack(">Q", stamp)[2:]
+#     rand_bytes = os.urandom(10)
+#     if stamp == prev_stamp and prev_rand is not None:
+#         while rand_bytes < prev_rand:
+#             nn += 1
+#             rand_bytes = os.urandom(10)
+#     prev_rand = rand_bytes
+#     prev_stamp = stamp
+#     return stamp, nn, stamp_bytes + rand_bytes
 
 # Courtesy of https://github.com/valohai/ulid2/blob/master/ulid2/__init__.py
-#
-# import time, calendar, struct
-# _last_entropy = None
-# _last_timestamp = None
-#
-# def generate_binary_ulid(timestamp=None, monotonic=False):
-#     """
-#     Generate the bytes for an ULID.
-#     :param timestamp: An optional timestamp override.
-#                       If `None`, the current time is used.
-#     :type timestamp: int|float|datetime.datetime|None
-#     :param monotonic: Attempt to ensure ULIDs are monotonically increasing.
-#                       Monotonic behavior is not guaranteed when used from multiple threads.
-#     :type monotonic: bool
-#     :return: Bytestring of length 16.
-#     :rtype: bytes
-#     """
-#     global _last_entropy, _last_timestamp
-#     if timestamp is None:
-#         timestamp = time.time()
-#     elif isinstance(timestamp, datetime.datetime):
-#         timestamp = calendar.timegm(timestamp.utctimetuple())
-#
-#     ts = int(timestamp * 1000.0)
-#     ts_bytes = struct.pack(b'!Q', ts)[2:]
-#     entropy = os.urandom(10)
-#     if monotonic and _last_timestamp == ts and _last_entropy is not None:
-#         while entropy < _last_entropy:
-#             entropy = os.urandom(10)
-#     _last_entropy = entropy
-#     _last_timestamp = ts
-#     return ts_bytes + entropy
-#
-
-
